@@ -1,5 +1,5 @@
 /*  GNU ddrescue - Data recovery tool
-    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
     Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
@@ -45,10 +45,15 @@ public:
     { if( size_ < 0 || size_ > LLONG_MAX - pos_ ) size_ = LLONG_MAX - pos_; }
   void end( const long long e ) throw()
     { pos_ = e - size_; if( pos_ < 0 ) { size_ += pos_; pos_ = 0; } }
-  void align_pos( const int hardbs ) throw();
-  void align_end( const int hardbs ) throw();
+  void align_pos( const int alignment ) throw();
+  void align_end( const int alignment ) throw();
   void inc_size( const long long delta ) throw() { size_ += delta; }
 
+  bool operator==( const Block & b ) const throw()
+    { return pos_ == b.pos_ && size_ == b.size_; }
+  bool operator!=( const Block & b ) const throw()
+    { return pos_ != b.pos_ || size_ != b.size_; }
+
   bool follows( const Block & b ) const throw()
     { return ( pos_ == b.end() ); }
   bool includes( const Block & b ) const throw()
@@ -80,6 +85,9 @@ public:
   Status status() const throw() { return status_; }
   void status( const Status st ) throw() { status_ = st; }
 
+  bool operator!=( const Sblock & sb ) const throw()
+    { return Block::operator!=( sb ) || status_ != sb.status_; }
+
   bool join( const Sblock & sb ) throw()
     { if( status_ == sb.status_ ) return Block::join( sb ); else return false; }
   Sblock split( const long long pos, const int hardbs = 1 )
@@ -87,6 +95,8 @@ public:
   static bool isstatus( const int st ) throw()
     { return ( st == non_tried || st == non_trimmed || st == non_split ||
                st == bad_sector || st == finished ); }
+  static bool is_good_status( const Status st ) throw()
+    { return ( st == non_tried || st == finished ); }
   };
 
 
@@ -95,7 +105,8 @@ class Domain
   std::vector< Block > block_vector;	// blocks are ordered and don't overlap
 
 public:
-  Domain( const char * const name, const long long p, const long long s );
+  Domain( const long long p, const long long s,
+          const char * const logname = 0 );
 
   long long pos() const throw()
     { if( block_vector.size() ) return block_vector[0].pos(); else return 0; }
@@ -119,6 +130,14 @@ public:
     { if( block_vector.size() ) return block_vector.back().end();
       else return 0; }
 
+  bool operator!=( const Domain & d ) const throw()
+    {
+    if( block_vector.size() != d.block_vector.size() ) return true;
+    for( unsigned int i = 0; i < block_vector.size(); ++i )
+      if( block_vector[i] != d.block_vector[i] ) return true;
+    return false;
+    }
+
   bool operator<( const Block & b ) const throw()
     { return ( block_vector.size() && block_vector.back().end() <= b.pos() ); }
 
@@ -151,4 +170,5 @@ public:
   void clear() { block_vector.clear(); }
   void crop( const Block & b );
   bool crop_by_file_size( const long long isize );
+  int blocks() const throw() { return (int)block_vector.size(); }
   };
